def keyword_only(*, a):
    pass

def keyword_only_default(*, a=42):
    pass

def arg_annotation(a:int):
    pass

def arg_annotation_default(a:int=42):
    pass

def arg_annotation_var(*args:int, **kwargs:str):
    pass

def arg_annotation_keyword_only(*, a:int):
    pass

def return_annotation() -> int:
    pass

def all_args(a:int, d=0, *args:int,
                g:int, h:int=8, **kwargs:int) -> int:
    return a, d, g, h, args, kwargs



import pep362

import unittest
from tests import pep362_fodder
try:
    from tests import pep362_py2_fodder
    from test import test_support
except SyntaxError:
    from tests import pep362_py3k_fodder
    from test import support as test_support
from sys import version_info


def version_specific(major_number):
    def inner(fxn):
        if version_info[0] == major_number:
            return fxn
        else:
            return lambda self: self
    return inner


class ParameterObjectTests(unittest.TestCase):

    """Test the Parameter object."""

    def test_name(self):
        # Test that 'name' attribute works.
        # Must test both using a string and a tuple of strings.
        name = "test"
        param = pep362.Parameter(name, 0)
        self.assertEqual(param.name, name)
        name = ('a', ('b',))
        param = pep362.Parameter(name, 0)
        self.assertEqual(param.name, name)

    def test_position(self):
        # Test the 'position' attribute.
        pos = 42
        param = pep362.Parameter("_", pos)
        self.assertEqual(param.position, pos)

    def test_default_values(self):
        # Testing that 'default_value' is not set is handled in the testing of
        # that attribute.
        default_value = 42
        param = pep362.Parameter('_', 0, True, default_value)
        self.assertEqual(param.default_value, default_value)
        param = pep362.Parameter('_', 0, False)
        self.assertTrue(not hasattr(param, 'default_value'))

    def test_keyword_only(self):
        # Setting the value for keyword_only should create an attribute.
        for value in (True, False):
            param = pep362.Parameter('_', 0, keyword_only=value)
            self.assertEqual(param.keyword_only, value)

    def test_annotations(self):
        # If has_annotation is False then 'annotation' should not exist.
        param = pep362.Parameter('_', 0, has_annotation=False)
        self.assertTrue(not hasattr(param, 'annotation'))
        annotation = 42
        param = pep362.Parameter('_', 0, has_annotation=True,
                                    annotation=annotation)
        self.assertEqual(param.annotation, annotation)


class SignatureObjectTests(unittest.TestCase):

    def test_getitem(self):
        # __getitem__() should return the Parameter object for the name
        # parameter.
        sig = pep362.Signature(pep362_fodder.default_args)
        self.assertTrue(sig['a'])
        param = sig['a']
        self.assertTrue(param.name, 'a')

    def test_iter(self):
        # The iterator should return all Parameter objects in the proper order.
        sig = pep362.Signature(pep362_fodder.default_args)
        params = list(sig)
        self.assertEqual(len(params), 1)
        self.assertEqual(params[0].name, 'a')

    def test_no_args(self):
        # Test a function with no arguments.
        sig = pep362.Signature(pep362_fodder.no_args)
        self.assertEqual('no_args', sig.name)
        self.assertTrue(not sig.var_args)
        self.assertTrue(not sig.var_kw_args)
        self.assertEqual(0, len(list(sig)))

    def test_var_args(self):
        # Test the var_args attribute.
        sig = pep362.Signature(pep362_fodder.var_args)
        self.assertEqual('args', sig.var_args)
        self.assertEqual(0, len(list(sig)))
        sig = pep362.Signature(pep362_fodder.no_args)
        self.assertEqual('', sig.var_args)

    def test_var_kw_args(self):
        # Test the var_kw_args attribute and annotations.
        sig = pep362.Signature(pep362_fodder.var_kw_args)
        self.assertEqual('var_kw_args', sig.name)
        self.assertEqual('kwargs', sig.var_kw_args)
        self.assertEqual(0, len(list(sig)))
        sig = pep362.Signature(pep362_fodder.no_args)
        self.assertEqual('', sig.var_kw_args)

    def test_parameter_positional(self):
        # A function with positional arguments should work.
        sig = pep362.Signature(pep362_fodder.no_default_args)
        self.assertEqual('no_default_args', sig.name)
        param = sig['a']
        self.assertEqual('a', param.name)
        self.assertEqual(0, param.position)
        self.assertTrue(not hasattr(param, 'default_value'))

    def test_parameter_default(self):
        # Default parameters for a function should work.
        sig = pep362.Signature(pep362_fodder.default_args)
        self.assertEqual('default_args', sig.name)
        param = sig['a']
        self.assertEqual('a', param.name)
        self.assertEqual(0, param.position)
        self.assertEqual(42, param.default_value)

    @version_specific(2)
    def test_parameter_tuple(self):
        # A function with a tuple as a parameter should work.
        sig = pep362.Signature(pep362_py2_fodder.tuple_args)
        self.assertEqual('tuple_args', sig.name)
        param = list(sig)[0]
        self.assertTrue(isinstance(param.name, tuple))
        self.assertEqual(('a', ('b',)), param.name)
        self.assertEqual(0, param.position)
        self.assertTrue(not hasattr(param, 'default_value'))

    @version_specific(2)
    def test_parameter_tuple_default(self):
        # A default argument for a tuple parameter needs to work.
        sig = pep362.Signature(pep362_py2_fodder.default_tuple_args)
        self.assertEqual('default_tuple_args', sig.name)
        param = list(sig)[0]
        self.assertEqual(('a', ('b',)), param.name)
        self.assertEqual(0, param.position)
        self.assertEqual((1, (2,)), param.default_value)

    @version_specific(3)
    def test_keyword_only(self):
        # Is a function containing keyword-only parameters handled properly?
        sig = pep362.Signature(pep362_py3k_fodder.keyword_only)
        param = sig['a']
        self.assertEqual(param.name, 'a')
        self.assertTrue(param.keyword_only)
        self.assertEqual(param.position, 0)

    @version_specific(3)
    def test_keyword_only_default(self):
        # Default arguments can work for keyword-only parameters.
        sig = pep362.Signature(pep362_py3k_fodder.keyword_only_default)
        param = sig['a']
        self.assertEqual(param.name, 'a')
        self.assertTrue(param.keyword_only)
        self.assertEqual(param.position, 0)
        self.assertEqual(param.default_value, 42)

    @version_specific(3)
    def test_annotations(self):
        # Make sure the proper annotation is found.
        sig = pep362.Signature(pep362_py3k_fodder.arg_annotation)
        param = sig['a']
        self.assertEqual(param.name, 'a')
        self.assertEqual(param.annotation, int)

    @version_specific(3)
    def test_annotations_default(self):
        # Annotations with a default value should work.
        sig = pep362.Signature(pep362_py3k_fodder.arg_annotation_default)
        param = sig['a']
        self.assertEqual(param.name, 'a')
        self.assertEqual(param.annotation, int)
        self.assertEqual(param.default_value, 42)

    @version_specific(3)
    def test_annotation_keyword_only(self):
        # Keyword-only parameters can have an annotation.
        sig = pep362.Signature(pep362_py3k_fodder.arg_annotation_keyword_only)
        param = sig['a']
        self.assertEqual(param.name, 'a')
        self.assertEqual(param.annotation, int)
        self.assertTrue(param.keyword_only)

    @version_specific(3)
    def test_return_annotation(self):
        # The return value annotation.
        sig = pep362.Signature(pep362_py3k_fodder.return_annotation)
        self.assertEqual(sig.return_annotation, int)

    @version_specific(3)
    def test_var_annotations(self):
        # Annotation on variable arguments (*args & **kwargs).
        sig = pep362.Signature(pep362_py3k_fodder.arg_annotation_var)
        self.assertEqual(sig.var_annotations[sig.var_args], int)
        self.assertEqual(sig.var_annotations[sig.var_kw_args], str)

    def test_signature(self):
        def fresh_func():
            pass
        self.assertTrue(not hasattr(fresh_func, '__signature__'))
        sig = pep362.signature(fresh_func)
        self.assertEqual(sig, fresh_func.__signature__)
        sig2 = pep362.signature(fresh_func)
        self.assertEqual(sig, sig2)
        class FreshClass(object):
            def fresh_method(self):
                pass
        sig = pep362.signature(FreshClass.fresh_method)
        self.assertEqual(sig, FreshClass.fresh_method.__signature__)


class SignatureBindTests(unittest.TestCase):

    """Test Signature.bind()."""

    def test_no_parameters(self):
        sig = pep362.Signature(pep362_fodder.no_args)
        binding = sig.bind()
        self.assertEqual({}, binding)
        self.assertRaises(pep362.BindError, sig.bind, 42)
        self.assertRaises(pep362.BindError, sig.bind, a=0)

    def test_var_parameters(self):
        sig = pep362.Signature(pep362_fodder.var_args)
        binding = sig.bind(0, 1, 2)
        self.assertEqual({'args':(0, 1, 2)}, binding)
        binding = sig.bind()
        self.assertEqual({'args':tuple()}, binding)
        self.assertRaises(pep362.BindError, sig.bind, a=0)

    def test_var_kw_parameters(self):
        sig = pep362.Signature(pep362_fodder.var_kw_args)
        binding = sig.bind(a=0)
        self.assertEqual({'kwargs':{'a':0}}, binding)
        binding = sig.bind()
        self.assertEqual({'kwargs':{}}, binding)
        self.assertRaises(pep362.BindError, sig.bind, 42)

    def test_positional_parameters(self):
        sig = pep362.Signature(pep362_fodder.no_default_args)
        binding = sig.bind(42)
        self.assertEqual({'a':42}, binding)
        binding = sig.bind(a=42)
        self.assertEqual({'a':42}, binding)
        self.assertRaises(pep362.BindError, sig.bind)
        self.assertRaises(pep362.BindError, sig.bind, 0, 1)
        self.assertRaises(pep362.BindError, sig.bind, b=0)

    def test_keyword_parameters(self):
        sig = pep362.Signature(pep362_fodder.default_args)
        binding = sig.bind()
        self.assertEqual({'a':42}, binding)
        binding = sig.bind(0)
        self.assertEqual({'a':0}, binding)
        binding = sig.bind(a=0)
        self.assertEqual({'a':0}, binding)
        self.assertRaises(pep362.BindError, sig.bind, 0, 1)
        self.assertRaises(pep362.BindError, sig.bind, a=0, b=1)
        self.assertRaises(pep362.BindError, sig.bind, b=1)

    @version_specific(2)
    def test_tuple_parameter(self):
        sig = pep362.Signature(pep362_py2_fodder.tuple_args)
        arg = (1, ((2,),))
        binding = sig.bind(arg)
        self.assertEqual({'a':1, 'b':(2,)}, binding)
        self.assertRaises(pep362.BindError, sig.bind, (1,2,3))
        self.assertRaises(pep362.BindError, sig.bind, (1, 2))

    @version_specific(2)
    def test_default_tuple_parameter(self):
        sig = pep362.Signature(pep362_py2_fodder.default_tuple_args)
        binding = sig.bind()
        self.assertEqual({'a':1, 'b':2}, binding)
        arg = (0, (1,))
        binding = sig.bind(arg)
        self.assertEqual({'a':0, 'b':1}, binding)

    @version_specific(2)
    def test_py2_all_args(self):
        sig = pep362.Signature(pep362_py2_fodder.all_args)
        # a, (b, (c,)), d=0, (e, (f,))=(4, (5,)), *g, **h
        # name, position, has_default, default value
        expect = (('a', 0, False, None),
                    (('b', ('c',)), 1, False, None),
                    ('d', 2, True, 0),
                    (('e', ('f',)), 3, True, (4, (5,))))
        self.assertEqual(len(list(sig)), len(expect))
        for param, check in zip(list(sig), expect):
            name, pos, has_default, default_value = check
            self.assertEqual(param.name, name)
            self.assertEqual(param.position, pos)
            if has_default:
                self.assertEqual(param.default_value, default_value)
            else:
                self.assertTrue(not hasattr(param, 'default_value'))
            self.assertTrue(not param.keyword_only)
            self.assertTrue(not hasattr(param, 'annotation'))
        self.assertEqual(sig.var_args, 'g')
        self.assertEqual(sig.var_kw_args, 'h')
        self.assertEqual(len(sig.var_annotations), 0)
        binding = sig.bind(0, (1, (2,)), d=3, i=7)
        expected = {'a':0, 'b':1, 'c':2, 'd':3, 'e':4, 'f':5, 'g':tuple(),
                    'h':{'i':7}}
        self.assertEqual(expected, binding)

    @version_specific(3)
    def test_keyword_only(self):
        sig = pep362.Signature(pep362_py3k_fodder.keyword_only)
        binding = sig.bind(a=42)
        self.assertEqual(binding, {'a':42})
        self.assertRaises(pep362.BindError, sig.bind)
        self.assertRaises(pep362.BindError, sig.bind, 42)

    @version_specific(3)
    def test_keyword_only_default(self):
        sig = pep362.Signature(pep362_py3k_fodder.keyword_only_default)
        binding = sig.bind()
        self.assertEqual(binding, {'a':42})
        binding = sig.bind(a=1)
        self.assertEqual(binding, {'a':1})
        self.assertRaises(pep362.BindError, sig.bind, 1)

    @version_specific(3)
    def test_all_py3k_args(self):
        # a:int, d=0, *args:int, g:int, h:int=8, **kwargs:int) -> int
        sig = pep362.Signature(pep362_py3k_fodder.all_args)
        # name, position, kw only, has_default, default, has anno, anno
        expected = (('a', 0, False, False, None, True, int),
                    ('d', 1, False, True, 0, False, None),
                    ('g', 2, True, False, None, True, int),
                    ('h', 3, True, True, 8, True, int))
        self.assertEqual(len(list(sig)), len(expected),
                "len(%r) != len(%r)" % ([param.name
                                            for param in sig],
                                        [expect[0] for expect in expected]))
        for param, check in zip(sig, expected):
            name, pos, kw_only, has_default, default, has_anno, anno = check
            self.assertEqual(param.name, name)
            self.assertEqual(param.position, pos)
            if kw_only:
                self.assertTrue(param.keyword_only)
            else:
                self.assertTrue(not param.keyword_only)
            if has_default:
                self.assertEqual(param.default_value, default)
            else:
                self.assertTrue(not hasattr(param, 'default_value'))
            if has_anno:
                self.assertEqual(param.annotation, anno)
            else:
                self.assertTrue(not hasattr(param, 'annotation'))
        self.assertEqual(sig.var_args, 'args')
        self.assertTrue(sig.var_args in sig.var_annotations)
        self.assertEqual(sig.var_annotations[sig.var_args], int)
        self.assertEqual(sig.var_kw_args, 'kwargs')
        self.assertTrue(sig.var_kw_args in sig.var_annotations)
        self.assertEqual(sig.var_annotations[sig.var_kw_args], int)
        self.assertEqual(sig.return_annotation, int)
        binding = sig.bind(0, 3, 6, g=7, i=9)
        expected = {'a':0, 'd':3, 'g':7, 'h':8, 'args':(6,), 'kwargs':{'i':9}}
        self.assertEqual(binding, expected)

    def test_too_many_arguments(self):
        # Only one argument should pair up with a parameter.
        sig = pep362.Signature(pep362_fodder.no_default_args)
        self.assertRaises(pep362.BindError, sig.bind, 1, a=1)


def test_main():
    test_support.run_unittest(ParameterObjectTests,
                                SignatureObjectTests,
                                SignatureBindTests,
                             )


if __name__ == '__main__':
    test_main()



from pep362 import Signature

def quack_check(fxn):
    """Decorator to verify arguments and return value quack as they should.

    Positional arguments.
    >>> @quack_check
    ... def one_arg(x:int): pass
    ... 
    >>> one_arg(42)
    >>> one_arg('a')
    Traceback (most recent call last):
        ...
    TypeError: 'a' does not quack like a <class 'int'>


    *args
    >>> @quack_check
    ... def var_args(*args:int): pass
    ... 
    >>> var_args(*[1,2,3])
    >>> var_args(*[1,'b',3])
    Traceback (most recent call last):
        ...
    TypeError: *args contains a a value that does not quack like a <class 'int'>

    **kwargs
    >>> @quack_check
    ... def var_kw_args(**kwargs:int): pass
    ... 
    >>> var_kw_args(**{'a': 1})
    >>> var_kw_args(**{'a': 'A'})
    Traceback (most recent call last):
        ...
    TypeError: **kwargs contains a value that does not quack like a <class 'int'>

    Return annotations.
    >>> @quack_check
    ... def returned(x) -> int: return x
    ... 
    >>> returned(42)
    42
    >>> returned('a')
    Traceback (most recent call last):
        ...
    TypeError: the return value 'a' does not quack like a <class 'int'>

    """
    # Get the signature; only needs to be calculated once.
    sig = Signature(fxn)
    def check(*args, **kwargs):
        # Find out the variable -> value bindings.
        bindings = sig.bind(*args, **kwargs)
        # Check *args for the proper quack.
        try:
            duck = sig.var_annotations[sig.var_args]
        except KeyError:
            pass
        else:
            # Check every value in *args.
            for value in bindings[sig.var_args]:
                if not isinstance(value, duck):
                    raise TypeError("*%s contains a a value that does not "
                                    "quack like a %r" %
                                    (sig.var_args, duck))
            # Remove it from the bindings so as to not check it again.
            del bindings[sig.var_args]
        # **kwargs.
        try:
            duck = sig.var_annotations[sig.var_kw_args]
        except (KeyError, AttributeError):
            pass
        else:
            # Check every value in **kwargs.
            for value in bindings[sig.var_kw_args].values():
                if not isinstance(value, duck):
                    raise TypeError("**%s contains a value that does not "
                                    "quack like a %r" %
                                    (sig.var_kw_args, duck))
            # Remove from bindings so as to not check again.
            del bindings[sig.var_kw_args]
        # For each remaining variable ...
        for var, value in bindings.items():
            # See if an annotation was set.
            try:
                duck = sig[var].annotation
            except AttributeError:
                continue
            # Check that the value quacks like it should.
            if not isinstance(value, duck):
                raise TypeError('%r does not quack like a %s' % (value, duck))
        else:
            # All the ducks quack fine; let the call proceed.
            returned = fxn(*args, **kwargs)
            # Check the return value.
            try:
                if not isinstance(returned, sig.return_annotation):
                    raise TypeError('the return value %r does not quack like '
                                    'a %r' % (returned,
                                        sig.return_annotation))
            except AttributeError:
                pass
            return returned
    # Full-featured version would set function metadata.
    return check


if __name__ == '__main__':
    import doctest
    doctest.testmod()



import inspect
from operator import attrgetter


class BindError(TypeError):
    """Represent a failure of inspect.Signature.bind() being able to to
    determine if a binding of arguments to parameters is possible."""
    pass


class Parameter(object):

    """Represent a parameter in a function signature.

    Each parameter has the following attributes:
    * name
        The name of the parameter.
    * position
        The position in the parameter list for the argument, no including any
        variable position argument.
    * keyword_only
        True if the parameter is keyword-only.
    
    And the following optoinal attributes:
    * default_value
        The default value for the parameter, if one exists.
    * annotation
        The annoation for the parameter, if one exists.

    """

    def __init__(self, name, position, has_default=False, default_value=None,
                 keyword_only=False, has_annotation=False, annotation=None):
        """Initialize a Parameter instance.

        For has_* arguments, if they are False then the corresponding *
        parameter is ignored.

        """
        self.name = name
        self.position = position
        if has_default:
            self.default_value = default_value
        self.keyword_only = keyword_only
        if has_annotation:
            self.annotation = annotation


class Signature(object):

    """Object to represent the signature of a function/method.

    Attributes:
    * name
        Name of the function/method.
    * var_args
        Name of the variable positional parameter, else ''.
    * var_kw_wargs
        Name of the variable keywrod parameter, else ''.
    * var_annotations
        Dict keyed on the variable parameter names with the values of the
        annotation for the parameter.  If an annotation does not exist for a
        parameter, the key does not exist.

    Optional attributes:
    * return_annotation
        The annotation for the return value.



    """

    def __init__(self, func):
        """Initialize from a function or method object."""
        if hasattr(func, 'im_func'):
            func = func.im_func
        try:
            func_code = func.__code__
        except AttributeError:
            # Compatibility for versions < 2.6.
            func_code = func.func_code

        self.name = func.__name__

        try:
            # Unneeded once 2.x support is removed; can easily get info the
            #  "hard" way.
            argspec = inspect.getfullargspec(func)[:4]
        except AttributeError:
            # Needed only for tuple parameters.
            argspec = inspect.getargspec(func)
        parameters = {}

        # Parameter information.
        pos_count = func_code.co_argcount
        if hasattr(func_code, 'co_kwonlyargcount'):
            keyword_only_count = func_code.co_kwonlyargcount
        else:
            keyword_only_count = 0
        positional = argspec[0]
        keyword_only = func_code.co_varnames[pos_count:
                                                pos_count+keyword_only_count]
        try:
            fxn_defaults = func.__defaults__
        except AttributeError:
            # Deal with old names prior to 2.6.
            fxn_defaults = func.func_defaults
        if fxn_defaults:
            pos_default_count = len(fxn_defaults)
        else:
            pos_default_count = 0

        # Non-keyword-only parameters w/o defaults.
        non_default_count = pos_count - pos_default_count
        for index, name in enumerate(positional[:non_default_count]):
            name = self._convert_name(name)
            has_annotation, annotation = self._find_annotation(func, name)
            param = Parameter(name, index, has_default=False,
                    has_annotation=has_annotation, annotation=annotation)
            parameters[name] = param
        # ... w/ defaults.
        for offset, name in enumerate(positional[non_default_count:]):
            name = self._convert_name(name)
            has_annotation, annotation = self._find_annotation(func, name)
            default_value = fxn_defaults[offset]
            param = Parameter(name, offset+non_default_count,
                                has_default=True, default_value=default_value,
                                has_annotation=has_annotation,
                                annotation=annotation)
            parameters[name] = param
        # Keyword-only parameters.
        for offset, name in enumerate(keyword_only):
            has_annotation, annotation = self._find_annotation(func, name)
            has_default, default_value = False, None
            # hasattr check only needed for versions < 2.6.
            if (hasattr(func, '__kwdefaults__') and func.__kwdefaults__ and
                    name in func.__kwdefaults__):
                has_default = True
                default_value = func.__kwdefaults__[name]
            param = Parameter(name, offset+pos_count, keyword_only=True,
                                has_default=has_default,
                                default_value=default_value,
                                has_annotation=has_annotation,
                                annotation=annotation)
            parameters[name] = param
        # Variable parameters.
        index = pos_count + keyword_only_count
        self.var_annotations = dict()
        if func_code.co_flags & 0x04:
            self.var_args = func_code.co_varnames[index]
            has_annotation, annotation = self._find_annotation(func,
                                                                self.var_args)
            if has_annotation:
                self.var_annotations[self.var_args] = (
                                        func.__annotations__[self.var_args])
            index += 1
        else:
            self.var_args = ''
        if func_code.co_flags & 0x08:
            self.var_kw_args = func_code.co_varnames[index]
            has_annotation, annotation = self._find_annotation(func,
                                                                self.var_kw_args)
            if has_annotation:
                self.var_annotations[self.var_kw_args] = (
                                    func.__annotations__[self.var_kw_args])
            index += 1
        else:
            self.var_kw_args = ''

        self._parameters = parameters

        # Return annotation.
        if hasattr(func, '__annotations__'):
            if 'return' in func.__annotations__:
                self.return_annotation = func.__annotations__['return']

    def __getitem__(self, key):
        return self._parameters[key]

    def __iter__(self):
        return iter(sorted(self._parameters.values(), key=attrgetter('position')))

    def _find_annotation(self, func, name):
        """Return True if an annotation exists for the named parameter along
        with its annotation, else return False and None."""
        has_annotation, annotation = False, None
        if hasattr(func, '__annotations__'):
            if name in func.__annotations__:
                has_annotation = True
                annotation = func.__annotations__[name]
        return has_annotation, annotation

    def _convert_name(self, name):
        if not isinstance(name, list):
            return name
        else:
            return tuple(self._convert_name(x) for x in name)

    def bind(self, *args, **kwargs):
        """Return a dictionary mapping function arguments to their parameter
        variables, if possible.
        
        Multiple arguments for the same parameter using keyword arguments
        cannot be detected.
       
        """
        bindings = {}
        if self.var_args:
            bindings[self.var_args] = tuple()
        if self.var_kw_args:
            bindings[self.var_kw_args] = dict()
        positional = []
        keyword_only = {}

        for param in self:
            if not param.keyword_only:
                positional.append(param)
            else:
                keyword_only[param.name] = param

        # Positional arguments.
        if not self._parameters and args and self.var_args:
            bindings[self.var_args] = args
            args = tuple()
        for index, position_arg in enumerate(args[:]):
            try:
                param = positional.pop(0)
            except IndexError:
                # *args.
                if self.var_args:
                    bindings[self.var_args] = tuple(args)
                    break
                else:
                    raise BindError("too many positional arguments")
            self._tuple_bind(bindings, param.name, position_arg)
            args = args[1:]
        # Keyword arguments & default values.
        else:
            for positional_param in positional:
                param_name = positional_param.name
                if param_name in kwargs:
                    try:
                        bindings[param_name] = kwargs[param_name]
                        del kwargs[param_name]
                    except KeyError:
                        raise BindError("%r unbound" % param_name)
                else:
                    if hasattr(positional_param, 'default_value'):
                        self._tuple_bind(bindings, param_name,
                                            positional_param.default_value)
                    else:
                        raise BindError("%r parameter lacking default value" %
                                        param_name)

        # Keyword arguments.
        positional_dict = dict((param.name, param) for param in positional)
        for key, value in kwargs.copy().items():
            if key in bindings:
                raise BindError("too many arguments for %r parameter"
                                % key)
            if key in positional_dict:
                del positional_dict[key]
            # Keyword-only.
            elif key in keyword_only:
                del keyword_only[key]
            # **kwargs.
            elif self.var_kw_args:
                    bindings[self.var_kw_args][key] = value
                    continue
            else:
                raise BindError("too many keyword arguments")
            bindings[key] = value
            del kwargs[key]
        # Keyword-only default values.
        else:
            for name, param in keyword_only.items():
                if hasattr(param, 'default_value'):
                    bindings[name] = param.default_value
                else:
                    raise BindError("%s parameter lacking a default value" %
                                    name)

        return bindings

    def _tuple_bind(self, bindings, possible_tuple, value):
        """Where a tuple could be a parameter, handle binding the values to the
        tuple and storing into the bindings mapping."""
        if not isinstance(possible_tuple, tuple):
            bindings[possible_tuple] = value
        else:
            # Need to make sure that value is as long as the parameter, but not
            # vice-versa.
            error_msg = "not enough values to unpack for %r"
            tuple_iter = iter(possible_tuple)
            try:
                value_iter = iter(value)
            except TypeError:
                raise BindError(error_msg % possible_tuple)
            while True:
                try:
                    sub_param = tuple_iter.next()
                except StopIteration:
                    break
                try:
                    sub_value = value_iter.next()
                except StopIteration:
                    raise BindError(error_msg % possible_tuple)
                self._tuple_bind(bindings, sub_param, sub_value)


def signature(func):
    """Return a Signature object for the function or method.

    If possible, return the existing value stored in __signature__.  If that
    attribute does not exist, then try to store the Signature object at that
    attribute if possible (but is not required).

    """
    if hasattr(func, 'im_func'):
        func = func.im_func
    sig = Signature(func)
    if not hasattr(func, '__signature__'):
        try:
            func.__signature__ = sig
        except AttributeError:
            pass
    else:
        sig = func.__signature__

    return sig



from distutils.core import setup

setup(
        # Package metadata.
        name='pep362',
        version='0.6.2',
        description='Implementation of PEP 362 (Function Signature objects)',
        author='Brett Cannon',
        author_email='brett@python.org',
        url='http://svn.python.org/view/sandbox/trunk/pep362/',
        # Files.
        py_modules=['pep362', 'examples'],
        packages=['tests'],
        data_files=['README'],
        classifiers=[
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.5',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.0',
            'Programming Language :: Python :: 3.1',
        ]
    )
